import random
import math
import pygame
import matplotlib.pyplot as plt
import numpy as np

pygame.init()

WIDTH, HEIGHT = 700, 400
WIN = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Brownsche Bewegung")

RED = (255, 82, 32)
BLACK = (0, 0, 0)
BLUE = (23, 2, 255)
WHITE = (255, 255, 255)
GREEN = (50, 205, 50)
YELLOW = (255, 255, 0)

SPEDING = 2

BALL_RADIUS = 4
BALL_AMOUNT = 50
BALL_COLOR = BLUE
BALL_MASSE = 1

BROWNSCHESTEILCHEN_MASSE = 50
BROWNSCHESTEILCHEN_RADIUS = 10
BROWNSCHESTEILCHEN_COLOR = RED

PLOT_RECHEN_SCHRITT = 120


class Ball:
    SPEEDING = SPEDING

    def __init__(self, radius, color, masse):
        # Startwert und aktuelle Pos
        self.x = self.original_x = random.randint(0 + radius, WIDTH - radius)
        self.y = self.original_y = random.randint(0 + radius, HEIGHT - radius)
        self.radius = radius
        # Richtungsvektor
        self.x_vel = random.uniform(-1, 1)
        self.y_vel = random.uniform(-1, 1)
        norm = math.sqrt(self.x_vel ** 2 + self.y_vel ** 2)
        self.x_vel = self.x_vel / norm
        self.y_vel = self.y_vel / norm
        self.masse = masse

        self.color = color

    def draw(self, win):
        pygame.draw.circle(win, self.color, (self.x, self.y), self.radius)

    def move(self):
        self.x += self.x_vel * self.SPEEDING
        self.y += self.y_vel * self.SPEEDING

    def handle_border_collision(self):
        if self.y + self.radius >= HEIGHT:
            self.y_vel *= -1
        elif self.y - self.radius <= 0:
            self.y_vel *= -1
        if self.x + self.radius >= WIDTH:
            self.x_vel *= -1
        elif self.x - self.radius <= 0:
            self.x_vel *= -1

    def handle_collision(self, b2):
        ab_x = b2.x - self.x
        ab_y = b2.y - self.y
        distance = math.sqrt(ab_x ** 2 + ab_y ** 2)
        if distance <= self.radius + b2.radius:

            #Stossvektor und co nötig?
            # stossvektor
            sv_x = b2.x - self.x
            sv_y = b2.y - self.y
            sv_x /= self.norm()
            sv_y /= self.norm()

            # parallelvektor Vb1
            vb1p_x = skalarprodukt((self.x_vel, self.y_vel), (sv_x, sv_y)) * sv_x
            vb1p_y = skalarprodukt((self.x_vel, self.y_vel), (sv_x, sv_y)) * sv_y
            # parallelvektor Vb2
            vb2p_x = skalarprodukt((b2.x_vel, b2.y_vel), (sv_x, sv_y)) * sv_x
            vb2p_y = skalarprodukt((b2.x_vel, b2.y_vel), (sv_x, sv_y)) * sv_y

            # senkrechtvektor Vb1
            vb1s_x = self.x_vel - vb1p_x
            vb1s_y = self.y_vel - vb1p_y
            # senkrechtvektor Vb2
            vb2s_x = b2.x_vel - vb2p_x
            vb2s_y = b2.y_vel - vb2p_y

            # nach dem stoß
            self.x_vel = vb2p_x + vb1s_x
            self.y_vel = vb2p_y + vb1s_y
            b2.x_vel = vb1p_x + vb2s_x
            b2.y_vel = vb1p_y + vb2s_y
            #nach dem stoß normiert
            #self.x_vel /= self.norm()
            #self.y_vel /= self.norm()
            #b2.x_vel /= b2.norm()
            #b2.y_vel /= b2.norm()

            #
            #self.x_vel = (self.masse * self.x_vel + b2.masse * (2 * b2.x_vel - self.x_vel)) / (self.masse + b2.masse)
            #self.y_vel = (self.masse * self.y_vel + b2.masse * (2 * b2.y_vel - self.y_vel)) / (self.masse + b2.masse)
            #b2.x_vel = (b2.masse * b2.x_vel + self.masse * (2 * self.x_vel - b2.x_vel)) / (self.masse + b2.masse)
            #b2.y_vel = (b2.masse * b2.y_vel + self.masse * (2 * self.y_vel - b2.y_vel)) / (self.masse + b2.masse)

    def norm(self):
        return math.sqrt(self.x_vel ** 2 + self.y_vel ** 2)


def draw(win, balls):
    win.fill(BLACK)

    for ball in balls:
        ball.draw(win)
    pygame.display.update()


def skalarprodukt(v1, v2):
    return v1[0] * v2[0] + v1[1] * v2[1]


def generate_balls(amount):
    balls = []
    for i in range(amount):
        ball = Ball(BALL_RADIUS, BALL_COLOR, BALL_MASSE)
        balls.append(ball)

    return balls


def main():
    run = 1
    brownsches_teilchen = Ball(BROWNSCHESTEILCHEN_RADIUS, BROWNSCHESTEILCHEN_COLOR, BROWNSCHESTEILCHEN_MASSE)

    balls = [brownsches_teilchen] + generate_balls(BALL_AMOUNT)
    vel_dict = {}

    k = 1
    while run:
        k += 1
        draw(WIN, balls)
        pygame.time.Clock().tick(60)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = 0
                break

        for ball in balls:
            ball.move()
            ball.handle_border_collision()
            for i in range(balls.index(ball) + 1, len(balls)):
                ball.handle_collision(balls[i])

            if k % PLOT_RECHEN_SCHRITT == 0:
                abselv = math.sqrt(ball.x_vel ** 2 + ball.y_vel ** 2)
                if abselv in vel_dict:
                    vel_dict[abselv] += 1
                else:
                    vel_dict[abselv] = 1

        for key in vel_dict:
            plt.bar(key, vel_dict[key], 0.2, color="blue")
        vel_dict = {}
        #plt.show()

    pygame.quit()


if __name__ == '__main__':
    main()
